package io.hackle.hackle;

import static io.hackle.hackle.Constants.Configs.DEBUG;
import static io.hackle.hackle.Constants.Configs.EXPOSURE_EVENT_DEDUP_INTERVAL_MILLIS;

import androidx.annotation.NonNull;

import java.util.HashMap;

import io.hackle.android.HackleConfig;
import io.hackle.sdk.common.Event;
import io.hackle.sdk.common.ParameterConfig;
import io.hackle.sdk.common.User;
import io.hackle.sdk.common.decision.Decision;
import io.hackle.sdk.common.decision.FeatureFlagDecision;

public class ModelConverter {
    User convertHashMapToUser(@NonNull HashMap<String, Object> map) {
        final String id = (String) map.get("id");
        final String deviceId = (String) map.get("deviceId");
        final String userId = (String) map.get("userId");

        final User.Builder builder = User.builder();

        if (id != null) {
            builder.id(id);
        }

        if (deviceId != null) {
            builder.deviceId(deviceId);
        }

        if (userId != null) {
            builder.userId(userId);
        }


        final HashMap<String, String> identifiers = (HashMap<String, String>) map.get("identifiers");
        for (HashMap.Entry<String, String> entry : identifiers.entrySet()) {
            builder.identifier(entry.getKey(), entry.getValue());
        }

        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");

        for (HashMap.Entry<String, Object> entry : properties.entrySet()) {
            final String propertyKey = entry.getKey();
            final Object value = entry.getValue();

            if (value instanceof String) {
                builder.property(propertyKey, (String) value);
            }

            if (value instanceof Integer) {
                builder.property(propertyKey, (Integer) value);
            }

            if (value instanceof Long) {
                builder.property(propertyKey, (Long) value);
            }

            if (value instanceof Double) {
                builder.property(propertyKey, (Double) value);
            }

            if (value instanceof Boolean) {
                builder.property(propertyKey, (Boolean) value);
            }
        }

        return builder.build();
    }

    HashMap<String, Object> convertUserToHashMap(@NonNull User user) {
        return new HashMap<String, Object>() {{
            put("id", user.getId());
            put("userId", user.getUserId());
            put("deviceId", user.getDeviceId());
            put("properties", user.getProperties());
            put("identifiers", user.getIdentifiers());
        }};
    }

    HackleConfig convertHashMapToHackleConfig(HashMap<String, Object> map, String wrapperName, String wrapperVersion) {
        final Object exposureEventDedupIntervalMillis = map.get(EXPOSURE_EVENT_DEDUP_INTERVAL_MILLIS);
        final Object debug = map.get(DEBUG);

        final HackleConfig.Builder builder = HackleConfig.builder()
                .add("$wrapper_name", wrapperName)
                .add("$wrapper_version", wrapperVersion);

        if (exposureEventDedupIntervalMillis instanceof Integer) {
            builder.exposureEventDedupIntervalMillis((Integer) exposureEventDedupIntervalMillis);
        }

        if (debug instanceof Boolean && (Boolean) debug) {
            builder.eventFlushIntervalMillis(1_000);
            builder.logLevel(3);
        }

        return builder.build();
    }

    Event convertHashMapToEvent(@NonNull HashMap<String, Object> map) {
        final String key = (String) map.get("key");
        final Double value = (Double) map.get("value");

        final Event.Builder builder = Event.builder(key);

        if (value != null) {
            builder.value(value);
        }

        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");
        for (HashMap.Entry<String, Object> entry : properties.entrySet()) {
            final String propertyKey = entry.getKey();
            final Object propertyValue = entry.getValue();

            if (propertyValue instanceof String) {
                builder.property(propertyKey, (String) propertyValue);
            }

            if (propertyValue instanceof Integer) {
                builder.property(propertyKey, (Integer) propertyValue);
            }

            if (propertyValue instanceof Long) {
                builder.property(propertyKey, (Long) propertyValue);
            }

            if (propertyValue instanceof Double) {
                builder.property(propertyKey, (Double) propertyValue);
            }

            if (propertyValue instanceof Boolean) {
                builder.property(propertyKey, (Boolean) propertyValue);
            }
        }

        return builder.build();
    }


    HashMap<String, Object> convertParameterConfigToHashMap(@NonNull final ParameterConfig config) {
        return new HashMap<String, Object>() {{
            put("parameters", new HashMap<>(config.getParameters()));
        }};
    }

    HashMap<String, Object> convertDecisionToHashMap(@NonNull final Decision decision) {
        return new HashMap<String, Object>() {{
            put("variation", decision.getVariation().toString());
            put("reason", decision.getReason().toString());
            put("config", convertParameterConfigToHashMap(decision.getConfig()));
        }};
    }

    HashMap<String, Object> convertFeatureFlagDecisionToHashMap(@NonNull final FeatureFlagDecision featureFlagDecision) {
        return new HashMap<String, Object>() {{
            put("isOn", featureFlagDecision.isOn());
            put("reason", featureFlagDecision.getReason().toString());
            put("config", convertParameterConfigToHashMap(featureFlagDecision.getConfig()));
        }};
    }
}

